@@ -1,4 +1,4 @@
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 # Name:        ipol
 # Purpose:
 #
@@ -7,7 +7,7 @@
 # Created:     26.10.2011
 # Copyright:   (c) Maik Heistermann, Stephan Jacobi and Thomas Pfaff 2011
 # Licence:     The MIT License
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 #!/usr/bin/env python
 
 """
@@ -39,6 +39,7 @@ This includes for example:
 
 """
 
+from functools import reduce
 import re
 import scipy
 from scipy.spatial import cKDTree
@@ -68,6 +69,7 @@ class IpolBase():
     def __init__(self, src, trg):
         src = self._make_coord_arrays(src)
         trg = self._make_coord_arrays(trg)
+
     def __call__(self, vals):
         """
         Evaluate interpolator for values given at the source points.
@@ -84,6 +86,7 @@ class IpolBase():
         """
         self._check_shape(vals)
         return None
+
     def _check_shape(self, vals):
         """
         Checks whether the values correspond to the source points
@@ -93,8 +96,9 @@ class IpolBase():
         vals : ndarray of float
 
         """
-        assert len(vals)==self.numsources, 'Length of value array %d does not correspond to \
+        assert len(vals) == self.numsources, 'Length of value array %d does not correspond to \
         number of source points %d' % (len(vals), self.numsources)
+
     def _make_coord_arrays(self, x):
         """
         Make sure that the coordinates are provided as ndarray
@@ -110,10 +114,10 @@ class IpolBase():
         if type(x) in [list, tuple]:
             x = [item.ravel() for item in x]
             x = np.array(x).transpose()
-        elif type(x)==np.ndarray:
-            if x.ndim==1:
-                x = x.reshape(-1,1)
-            elif x.ndim==2:
+        elif type(x) == np.ndarray:
+            if x.ndim == 1:
+                x = x.reshape(-1, 1)
+            elif x.ndim == 2:
                 pass
             else:
                 raise Exception('Cannot deal wih 3-d arrays, yet.')
@@ -139,12 +143,11 @@ class IpolBase():
         if vals.ndim < 2:
             # ndmin might be 0 so we get it to 1-d first
             # then we add an axis as we assume that
-            return np.atleast_1d(vals)[:,np.newaxis]
+            return np.atleast_1d(vals)[:, np.newaxis]
         else:
             return vals
 
 
-
 class Nearest(IpolBase):
     """
     Nearest(src, trg)
@@ -158,9 +161,9 @@ class Nearest(IpolBase):
     trg : ndarray of floats, shape (npoints, ndims)
         Data point coordinates of the target points.
 
-    Notes
-    -----
-    Uses ``scipy.spatial.cKDTree``
+    Note
+    ----
+    Uses :func:`scipy.spatial.cKDTree`
 
     """
 
@@ -173,6 +176,7 @@ class Nearest(IpolBase):
         # plant a tree
         self.tree = cKDTree(src)
         self.dists, self.ix = self.tree.query(trg, k=1)
+
     def __call__(self, vals, maxdist=None):
         """
         Evaluate interpolator for values given at the source points.
@@ -192,10 +196,10 @@ class Nearest(IpolBase):
         """
         self._check_shape(vals)
         out = vals[self.ix]
-        if maxdist==None:
+        if maxdist == None:
             return out
         else:
-            return np.where(self.dists>maxdist, np.nan, out)
+            return np.where(self.dists > maxdist, np.nan, out)
 
 
 class Idw(IpolBase):
@@ -213,9 +217,9 @@ class Idw(IpolBase):
     nnearest : integer - max. number of neighbours to be considered
     p : float - inverse distance power used in 1/dist**p
 
-    Notes
-    -----
-    Uses ``scipy.spatial.cKDTree``
+    Note
+    ----
+    Uses :func:`scipy.spatial.cKDTree`
 
     """
 
@@ -232,9 +236,8 @@ class Idw(IpolBase):
         self.dists, self.ix = self.tree.query(trg, k=nnearest)
         # avoid bug, if there is only one neighbor at all
         if self.dists.ndim == 1:
-            self.dists = self.dists[:,np.newaxis]
-            self.ix = self.ix[:,np.newaxis]
-
+            self.dists = self.dists[:, np.newaxis]
+            self.ix = self.ix[:, np.newaxis]
 
     def __call__(self, vals):
         """
@@ -255,16 +258,16 @@ class Idw(IpolBase):
         """
 
         # self distances: a list of arrays of distances of the nearest points which are indicated by self.ix
-        outshape = list( vals.shape )
+        outshape = list(vals.shape)
         outshape[0] = len(self.dists)
         interpol = np.repeat(np.nan, util._shape2size(outshape)).reshape(tuple(outshape)).astype('f4')
         # weights is the container for the weights (a list)
-        weights  = range( len(self.dists) )
+        weights = list(range(len(self.dists)))
         # sources is the container for the source point indices
-        src_ix   = range( len(self.dists) )
+        src_ix = list(range(len(self.dists)))
         # jinterpol is the jth element of interpol
         jinterpol = 0
-        for dist, ix in zip( self.dists, self.ix ):
+        for dist, ix in zip(self.dists, self.ix):
             valid_dists = np.where(np.isfinite(dist))[0]
             dist = dist[valid_dists]
             ix = ix[valid_dists]
@@ -275,17 +278,17 @@ class Idw(IpolBase):
             elif dist[0] < 1e-10:
                 # if a target point coincides with a source point
                 wz = vals[ix[0]]
-                w  = 1.
+                w = 1.
             else:
                 # weight z values by (1/dist)**p --
-                w = 1. / dist**self.p
+                w = 1. / dist ** self.p
                 w /= np.sum(w)
-                wz = np.dot( w, vals[ix] )
+                wz = np.dot(w, vals[ix])
             interpol[jinterpol] = wz.ravel()
-            weights [jinterpol] = w
-            src_ix  [jinterpol] = ix
+            weights[jinterpol] = w
+            src_ix[jinterpol] = ix
             jinterpol += 1
-        return interpol ## if self.qdim > 1  else interpol[0]
+        return interpol  ## if self.qdim > 1  else interpol[0]
 
 
 class Linear(IpolBase):
@@ -301,7 +304,6 @@ class Linear(IpolBase):
         Data point coordinates of the source points.
     trg : ndarray of floats, shape (npoints, ndims)
         Data point coordinates of the target points.
-
     """
 
     def __init__(self, src, trg):
@@ -310,6 +312,7 @@ class Linear(IpolBase):
         # remember some things
         self.numtargets = len(trg)
         self.numsources = len(src)
+
     def __call__(self, vals, fill_value=np.nan):
         """
         Evaluate interpolator for values given at the source points.
@@ -331,19 +334,19 @@ class Linear(IpolBase):
         return ip(self.trg)
 
 
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 # Covariance routines needed for Kriging
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 def parse_covariogram(cov_model):
     """"""
-    patterns = [re.compile('([\d\.]+) Nug\(([\d\.]+)\)'), # nugget
-                re.compile('([\d\.]+) Lin\(([\d\.]+)\)'), # linear
-                re.compile('([\d\.]+) Sph\(([\d\.]+)\)'), # spherical
-                re.compile('([\d\.]+) Exp\(([\d\.]+)\)'), # exponential
-                re.compile('([\d\.]+) Gau\(([\d\.]+)\)'), # gaussian
-                re.compile('([\d\.]+) Mat\(([\d\.]+)\)\^([\d\.]+)'), #matern
-                re.compile('([\d\.]+) Pow\(([\d\.]+)\)'),#power
-                re.compile('([\d\.]+) Cau\(([\d\.]+)\)\^([\d\.]+)\^([\d\.]+)'),# cauchy
+    patterns = [re.compile('([\d\.]+) Nug\(([\d\.]+)\)'),  # nugget
+                re.compile('([\d\.]+) Lin\(([\d\.]+)\)'),  # linear
+                re.compile('([\d\.]+) Sph\(([\d\.]+)\)'),  # spherical
+                re.compile('([\d\.]+) Exp\(([\d\.]+)\)'),  # exponential
+                re.compile('([\d\.]+) Gau\(([\d\.]+)\)'),  # gaussian
+                re.compile('([\d\.]+) Mat\(([\d\.]+)\)\^([\d\.]+)'),  # matern
+                re.compile('([\d\.]+) Pow\(([\d\.]+)\)'),  # power
+                re.compile('([\d\.]+) Cau\(([\d\.]+)\)\^([\d\.]+)\^([\d\.]+)'),  # cauchy
                 ]
 
     cov_funs = [cov_nug,
@@ -384,31 +387,31 @@ def cov_nug(h, sill, rng):
     Therefore, usually rng is set to 0
     """
     h = np.asanyarray(h)
-    return np.where(h<=rng, sill, 0.)
+    return np.where(h <= rng, sill, 0.)
 
 
 def cov_exp(h, sill=1.0, rng=1.0):
     """exponential type covariance function"""
     h = np.asanyarray(h)
-    return sill * (np.exp(-h/rng))
+    return sill * (np.exp(-h / rng))
 
 
 def cov_sph(h, sill=1.0, rng=1.0):
     """spherical type covariance function"""
     h = np.asanyarray(h)
-    return np.where(h<rng, sill * (1. - 1.5*h/rng + h**3/(2*rng**3)), 0.)
+    return np.where(h < rng, sill * (1. - 1.5 * h / rng + h ** 3 / (2 * rng ** 3)), 0.)
 
 
 def cov_gau(h, sill=1.0, rng=1.0):
     """gaussian type covariance function"""
     h = np.asanyarray(h)
-    return sill * np.exp(-h**2/rng**2)
+    return sill * np.exp(-h ** 2 / rng ** 2)
 
 
 def cov_lin(h, sill=1.0, rng=1.0):
     """linear covariance function"""
     h = np.asanyarray(h)
-    return np.where(h<rng, sill * (-h/rng + 1.), 0.)
+    return np.where(h < rng, sill * (-h / rng + 1.), 0.)
 
 
 def cov_mat(h, sill=1.0, rng=1.0, shp=0.5):
@@ -423,13 +426,13 @@ def cov_mat(h, sill=1.0, rng=1.0, shp=0.5):
     if shp > 100:
         c = cov_gau(h, sill, rng)
     else:
-        Kv = scipy.special.kv      # modified bessel function of second kind of order v
+        Kv = scipy.special.kv  # modified bessel function of second kind of order v
         Tau = scipy.special.gamma  # Gamma function
 
-        fac1 = h / rng * 2.0*np.sqrt(shp)
-        fac2 = (Tau(shp)*2.0**(shp-1.0))
+        fac1 = h / rng * 2.0 * np.sqrt(shp)
+        fac2 = (Tau(shp) * 2.0 ** (shp - 1.0))
 
-        c = np.where(h!=0, sill * 1.0 / fac2 * fac1**shp * Kv(shp, fac1), sill)
+        c = np.where(h != 0, sill * 1.0 / fac2 * fac1 ** shp * Kv(shp, fac1), sill)
 
     return c
 
@@ -437,7 +440,7 @@ def cov_mat(h, sill=1.0, rng=1.0, shp=0.5):
 def cov_pow(h, sill=1.0, rng=1.0):
     """power law covariance function"""
     h = np.asanyarray(h)
-    return sill - h**rng
+    return sill - h ** rng
 
 
 def cov_cau(h, sill=1., rng=1., alpha=1., beta=1.0):
@@ -448,7 +451,7 @@ def cov_cau(h, sill=1., rng=1., alpha=1., beta=1.0):
     beta >0 ... parameterizes long term memory
     """
     h = np.asanyarray(h).astype('float')
-    return sill*(1 + (h/rng)**alpha)**(-beta/alpha)
+    return sill * (1 + (h / rng) ** alpha) ** (-beta / alpha)
 
 
 class OrdinaryKriging(IpolBase):
@@ -487,18 +490,15 @@ class OrdinaryKriging(IpolBase):
     ----------
     src : ndarray of floats, shape (npoints, ndims)
         Data point coordinates of the source points.
-
     trg : ndarray of floats, shape (npoints, ndims)
         Data point coordinates of the target points.
-
     cov : string
         covariance (variogram) model string in the syntax ``gstat``
         uses.
-
     nnearest : integer - max. number of neighbours to be considered
 
-    Notes
-    -----
+    Note
+    ----
     The class calculates the Kriging weights during initialization, because
     these only depend on the configuration of the points.
 
@@ -511,6 +511,7 @@ class OrdinaryKriging(IpolBase):
     may be retrieved from the attribute `estimation_variance`.
 
     """
+
     def __init__(self, src, trg, cov='1.0 Exp(10000.)', nnearest=12):
         """"""
         self.src = self._make_coord_arrays(src)
@@ -521,11 +522,11 @@ class OrdinaryKriging(IpolBase):
         self.nnearest = nnearest
         # plant a tree
         self.tree = cKDTree(src)
-        self.dists, self.ix = self.tree.query(trg, k=min(nnearest,self.numsources))
+        self.dists, self.ix = self.tree.query(trg, k=min(nnearest, self.numsources))
         # avoid bug, if there is only one neighbor at all
         if self.dists.ndim == 1:
-            self.dists = self.dists[:,np.newaxis]
-            self.ix = self.ix[:,np.newaxis]
+            self.dists = self.dists[:, np.newaxis]
+            self.ix = self.ix[:, np.newaxis]
         # parse covariogram function string
         self.cov_func = parse_covariogram(cov)
         self.weights = []
@@ -533,20 +534,18 @@ class OrdinaryKriging(IpolBase):
         # do the kriging
         self._krige()
 
-
     def _krig_matrix(self, src):
         """Sets up the kriging system for a configuration of source points.
         """
         var_matrix = self.cov_func(scipy.spatial.distance_matrix(src, src))
 
-        ok_matrix = np.ones((len(src)+1, len(src)+1))
+        ok_matrix = np.ones((len(src) + 1, len(src) + 1))
 
-        ok_matrix[:-1,:-1] = var_matrix
-        ok_matrix[-1,-1] = 0.
+        ok_matrix[:-1, :-1] = var_matrix
+        ok_matrix[-1, -1] = 0.
 
         return ok_matrix
 
-
     def _krig_rhs(self, dists):
         """Sets up a right hand side of the kriging system given the distances
         of the target to the source points. To be used in conjunction with
@@ -556,19 +555,17 @@ class OrdinaryKriging(IpolBase):
 
         return ok_rhs
 
-
     def _krige(self):
         """Sets up the kriging system and solves it in order to obtain the
         interpolation weights of ordinary kriging.
         Also calculates the kriging estimation variance from the results"""
         for dist, ix in zip(self.dists, self.ix):
-            matrix = self._krig_matrix(self.src[ix,:])
+            matrix = self._krig_matrix(self.src[ix, :])
             rhs = self._krig_rhs(dist)
             weights = np.linalg.solve(matrix, rhs)
             self.weights.append(weights)
             self.estimation_variance.append(self.cov_func(0.)
-                                            - np.sum(weights*rhs))
-
+                                            - np.sum(weights * rhs))
 
     def __call__(self, vals):
         """
@@ -592,7 +589,7 @@ class OrdinaryKriging(IpolBase):
         self._check_shape(v)
         # calculate estimator
         weights = np.array(self.weights)
-        ip = np.add.reduce(weights[:,:-1,np.newaxis]*v[self.ix,...], axis=1)
+        ip = np.add.reduce(weights[:, :-1, np.newaxis] * v[self.ix, ...], axis=1)
 
         return ip
 
@@ -608,19 +605,15 @@ class ExternalDriftKriging(IpolBase):
     ----------
     src : ndarray of floats, shape (nsrcpoints, ndims)
         Data point coordinates of the source points.
-
     trg : ndarray of floats, shape (ntrgpoints, ndims)
         Data point coordinates of the target points.
-
     cov : string
         covariance (variogram) model string in the syntax ``gstat``
         uses.
-
-    nnearest : integer - max. number of neighbours to be considered
-
+    nnearest : int
+        max. number of neighbours to be considered
     src_drift : ndarray of floats, shape (nsrcpoints,)
         values of the external drift at each source point
-
     trg_drift : ndarray of floats, shape (ntrgpoints,)
         values of the external drift at each target point
 
@@ -628,8 +621,8 @@ class ExternalDriftKriging(IpolBase):
     --------
     OrdinaryKriging
 
-    Notes
-    -----
+    Note
+    ----
     After calling the object in order to get the interpolated values,
     the estimation variance of the system may be
     retrieved from the attribute `estimation_variance`. Accordingly, the
@@ -641,6 +634,7 @@ class ExternalDriftKriging(IpolBase):
     by passing new values to the __call__ method.
 
     """
+
     def __init__(self, src, trg, cov='1.0 Exp(10000.)', nnearest=12,
                  src_drift=None, trg_drift=None):
         """"""
@@ -654,39 +648,37 @@ class ExternalDriftKriging(IpolBase):
         self.nnearest = nnearest
         # plant a tree
         self.tree = cKDTree(src)
-        self.dists, self.ix = self.tree.query(trg, k=min(nnearest,self.numsources))
+        self.dists, self.ix = self.tree.query(trg, k=min(nnearest, self.numsources))
         # avoid bug, if there is only one neighbor at all
         if self.dists.ndim == 1:
-            self.dists = self.dists[:,np.newaxis]
-            self.ix = self.ix[:,np.newaxis]
+            self.dists = self.dists[:, np.newaxis]
+            self.ix = self.ix[:, np.newaxis]
         # parse covariogram function string
         self.cov_func = parse_covariogram(cov)
         self.weights = []
         self.estimation_variance = []
 
-
     def _krig_matrix(self, src, drift):
         """Sets up the kriging system for a configuration of source points.
         """
         # the basic covariance matrix
         var_matrix = self.cov_func(scipy.spatial.distance_matrix(src, src))
         # the extended matrix, initialized to ones
-        edk_matrix = np.ones((len(src)+2, len(src)+2))
+        edk_matrix = np.ones((len(src) + 2, len(src) + 2))
 
         # adding entries for the first lagrange multiplier for the ordinary
         # kriging part
-        edk_matrix[:-2,:-2] = var_matrix
-        edk_matrix[-2,-2] = 0.
+        edk_matrix[:-2, :-2] = var_matrix
+        edk_matrix[-2, -2] = 0.
 
         # adding entries for the second lagrange multiplier for the  edk part
-        edk_matrix[:-2,-1] = drift
-        edk_matrix[-1,:-2] = drift
-        edk_matrix[-2:,-1] = 0.
-        edk_matrix[-1,-2:] = 0.
+        edk_matrix[:-2, -1] = drift
+        edk_matrix[-1, :-2] = drift
+        edk_matrix[-2:, -1] = 0.
+        edk_matrix[-1, -2:] = 0.
 
         return edk_matrix
 
-
     def _krig_rhs(self, dists, drift):
         """Sets up a right hand side of the kriging system given the distances
         of the target to the source points. To be used in conjunction with
@@ -696,7 +688,6 @@ class ExternalDriftKriging(IpolBase):
 
         return edk_rhs
 
-
     def _krige(self, src_drift, trg_drift):
         """Sets up the kriging system and solves it in order to obtain the
         interpolation weights of ordinary kriging.
@@ -704,7 +695,7 @@ class ExternalDriftKriging(IpolBase):
         all_weights = []
         estimation_variances = []
         for dist, ix, td in zip(self.dists, self.ix, trg_drift):
-            matrix = self._krig_matrix(self.src[ix,:], src_drift[ix])
+            matrix = self._krig_matrix(self.src[ix, :], src_drift[ix])
             rhs = self._krig_rhs(dist, td)
             try:
                 weights = np.linalg.solve(matrix, rhs)
@@ -712,11 +703,10 @@ class ExternalDriftKriging(IpolBase):
                 weights = np.repeat(np.nan, len(rhs))
             all_weights.append(weights)
             estimation_variances.append(self.cov_func(0.)
-                                        - np.sum(weights*rhs))
+                                        - np.sum(weights * rhs))
 
         return all_weights, estimation_variances
 
-
     def __call__(self, vals, src_drift=None, trg_drift=None):
         """
         Evaluate interpolator for values given at the source points.
@@ -742,14 +732,14 @@ class ExternalDriftKriging(IpolBase):
         if src_drift is None:
             # check if we have data from __init__
             if self.src_drift is None:
-                raise ValueError, 'src_drift must be specified either on ' \
-                   + 'initialization or when calling the interpolator.'
+                raise ValueError('src_drift must be specified either on ' \
+                                 + 'initialization or when calling the interpolator.')
             src_drift = self.src_drift
         if trg_drift is None:
             # check if we have data from __init__
             if self.trg_drift is None:
-                raise ValueError, 'trg_drift must be specified either on ' \
-                   + 'initialization or when calling the interpolator.'
+                raise ValueError('trg_drift must be specified either on ' \
+                                 + 'initialization or when calling the interpolator.')
             trg_drift = self.trg_drift
 
         src_d = self._make_2d(src_drift)
@@ -768,33 +758,31 @@ class ExternalDriftKriging(IpolBase):
             self.weights = wght
             self.estimation_variance = variances
             weights = np.array(self.weights)
-            ip = np.add.reduce(weights[:,:-2,np.newaxis]*v[self.ix,...], axis=1)
+            ip = np.add.reduce(weights[:, :-2, np.newaxis] * v[self.ix, ...], axis=1)
         # otherwise we need to setup and solve the kriging system for each
         # field individually
         else:
             ip = np.empty((self.trg.shape[0], v.shape[1]))
             assert (v.shape[1] == src_d.shape[1]) and (v.shape[1] == trg_d.shape[1])
             for i in range(v.shape[1]):
-                wght, variances = self._krige(src_d[:,i].squeeze(),
-                                              trg_d[:,i].squeeze())
+                wght, variances = self._krige(src_d[:, i].squeeze(),
+                                              trg_d[:, i].squeeze())
 
                 weights = np.array(wght)
-                ip[:,i] = np.add.reduce(weights[:,:-2]*v[self.ix,i], axis=1)
+                ip[:, i] = np.add.reduce(weights[:, :-2] * v[self.ix, i], axis=1)
                 self.weights.append(weights)
                 self.estimation_variance.append(variances)
 
         return ip
 
 
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 # Wrapper functions
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 def interpolate(src, trg, vals, Interpolator, *args, **kwargs):
     """
     Convenience function to use the interpolation classes in an efficient way
 
-    ATTENTION: Works only for one- and two-dimensional *vals* arrays, yet.
-
     The interpolation classes in wradlib.ipol are computationally very efficient
     if they are applied on large multi-dimensional arrays of which the first dimension
     must be the locations' dimension (1d or 2d coordinates) and the following dimensions
@@ -816,6 +804,10 @@ def interpolate(src, trg, vals, Interpolator, *args, **kwargs):
     any *np.nan* values OR you have to post-process missing values in your interpolation
     result in another way.
 
+    Warning
+    -------
+    Works only for one- and two-dimensional *vals* arrays, yet.
+
     Parameters
     ----------
     src : ndarray of floats, shape (npoints, ndims)
@@ -825,7 +817,13 @@ def interpolate(src, trg, vals, Interpolator, *args, **kwargs):
     vals : ndarray of float, shape (numsourcepoints, ...)
         Values at the source points which to interpolate
     Interpolator : a class which inherits from IpolBase
+
+    Other Parameters
+    ----------------
     *args : arguments of Interpolator (see class documentation)
+
+    Keyword Arguments
+    -----------------
     **kwargs : keyword arguments of Interpolator (see class documentation)
 
     Examples
@@ -843,33 +841,35 @@ def interpolate(src, trg, vals, Interpolator, *args, **kwargs):
     >>> line2 = plt.plot(src, vals, 'ro')
 
     """
-    if vals.ndim==1:
+    if vals.ndim == 1:
         # source values are one dimensional, we have just to remove invalid data
         ix_valid = np.where(np.isfinite(vals))[0]
         ip = Interpolator(src[ix_valid], trg, *args, **kwargs)
         result = ip(vals[ix_valid])
-    elif vals.ndim==2:
+    elif vals.ndim == 2:
         # this implementation for 2 dimensions needs generalization
         ip = Interpolator(src, trg, *args, **kwargs)
         result = ip(vals)
         nan_in_result = np.where(np.isnan(result))
         nan_in_vals = np.where(np.isnan(vals))
         for i in np.unique(nan_in_result[-1]):
-            ix_good = np.where(np.isfinite(vals[...,i]))[0]
-            ix_broken_targets = nan_in_result[0][np.where(nan_in_result[-1]==i)[0]]
-            ip = Interpolator(src[ix_good], trg[nan_in_result[0][np.where(nan_in_result[-1]==i)[0]]], *args, **kwargs)
-            tmp = ip(vals[ix_good,i].reshape((len(ix_good),-1)))
-            result[ix_broken_targets,i] = tmp.ravel()
+            ix_good = np.where(np.isfinite(vals[..., i]))[0]
+            ix_broken_targets = nan_in_result[0][np.where(nan_in_result[-1] == i)[0]]
+            ip = Interpolator(src[ix_good], trg[nan_in_result[0][np.where(nan_in_result[-1] == i)[0]]], *args, **kwargs)
+            tmp = ip(vals[ix_good, i].reshape((len(ix_good), -1)))
+            result[ix_broken_targets, i] = tmp.ravel()
     else:
         if not np.any(np.isnan(vals.ravel())):
-            raise Exception('At the moment, <interpolate> can only deal with NaN values in <vals> if vals has less than 3 dimension.')
+            raise Exception(
+                'At the moment, <interpolate> can only deal with NaN values in <vals> if vals has less than 3 dimension.')
         else:
             # if no NaN value are in <vals> we can safely apply the Interpolator as is
             ip = Interpolator(src, trg, *args, **kwargs)
             result = ip(vals[ix_valid])
     return result
 
-def interpolate_polar(data, mask = None, Interpolator = Nearest):
+
+def interpolate_polar(data, mask=None, Interpolator=Nearest):
     """
     Convenience function to interpolate polar data
 
@@ -907,10 +907,10 @@ def interpolate_polar(data, mask = None, Interpolator = Nearest):
 
 
     """
-    if mask==None:
+    if mask is None:
         # no mask assigned: try to get it from masked array
         if type(data) != np.ma.core.MaskedArray:
-            print 'Warning! Neither an explicit mask is assigned nor the data-array is masked.'
+            print('Warning! Neither an explicit mask is assigned nor the data-array is masked.')
         mask = np.ma.getmaskarray(data)
     elif not np.any(mask):
         # mask contains no True values, so there is nothing to fill
@@ -924,11 +924,11 @@ def interpolate_polar(data, mask = None, Interpolator = Nearest):
     binx = np.cos(angles) * ranges
     biny = np.sin(angles) * ranges
     # calculate cartesian coordinates for bins, which are not masked
-    src_coord = np.array([(np.delete(binx,clutter_indices)), (np.delete(biny,clutter_indices))]).transpose()
+    src_coord = np.array([(np.delete(binx, clutter_indices)), (np.delete(biny, clutter_indices))]).transpose()
     # calculate cartesian coordinates for bins, which are masked
     trg_coord = np.array([binx[clutter_indices], biny[clutter_indices]]).transpose()
     # data values for bins, which are not masked
-    values_list = np.delete(data,clutter_indices)
+    values_list = np.delete(data, clutter_indices)
     filled_data = data.copy().ravel()
     # interpolate masked bins
     filling = interpolate(src_coord, trg_coord, values_list, Interpolator)
@@ -938,9 +938,9 @@ def interpolate_polar(data, mask = None, Interpolator = Nearest):
     # by nearest Neighbor interpolation
     if np.any(np.isnan(filled_data)):
         trg_coord = np.array([binx[np.where(np.isnan(filled_data))], biny[np.where(np.isnan(filled_data))]]).transpose()
-        filling = interpolate(src_coord, trg_coord, values_list, Interpolator = Nearest)
+        filling = interpolate(src_coord, trg_coord, values_list, Interpolator=Nearest)
         filled_data[np.where(np.isnan(filled_data))] = filling
-    return filled_data.reshape(data.shape[0],data.shape[1])
+    return filled_data.reshape(data.shape[0], data.shape[1])
 
 
 def cart2irregular_interp(cartgrid, values, newgrid, **kwargs):
@@ -986,6 +986,7 @@ def cart2irregular_interp(cartgrid, values, newgrid, **kwargs):
 
     return interp
 
+
 def cart2irregular_spline(cartgrid, values, newgrid, **kwargs):
     """
     Map array ``values`` defined by cartesian coordinate array ``cartgrid``
@@ -1013,26 +1014,25 @@ def cart2irregular_spline(cartgrid, values, newgrid, **kwargs):
     """
 
     # TODO: dimension checking
-
     newshape = newgrid.shape[:-1]
 
-    xi = newgrid[...,0].ravel()
-    yi = newgrid[...,1].ravel()
+    xi = newgrid[..., 0].ravel()
+    yi = newgrid[..., 1].ravel()
 
     nx = cartgrid.shape[1]
     ny = cartgrid.shape[0]
 
-    cxmin = np.min(cartgrid[...,0])
-    cxmax = np.max(cartgrid[...,0])
-    cymin = np.min(cartgrid[...,1])
-    cymax = np.max(cartgrid[...,1])
+    cxmin = np.min(cartgrid[..., 0])
+    cxmax = np.max(cartgrid[..., 0])
+    cymin = np.min(cartgrid[..., 1])
+    cymax = np.max(cartgrid[..., 1])
 
     # this functionality finds the floating point
     # indices into the value array (0:nx-1)
     # can be transferred into separate function
     # if necessary
-    xi = (nx - 1) * (xi - cxmin) / (cxmax-cxmin)
-    yi = (ny - 1) * (yi - cymin) / (cymax-cymin)
+    xi = (nx - 1) * (xi - cxmin) / (cxmax - cxmin)
+    yi = (ny - 1) * (yi - cymin) / (cymax - cymin)
 
     # interpolation by map_coordinates
     interp = map_coordinates(values, [yi, xi], **kwargs)
@@ -1042,8 +1042,4 @@ def cart2irregular_spline(cartgrid, values, newgrid, **kwargs):
 
 
 if __name__ == '__main__':
-    print 'wradlib: Calling module <ipol> as main...'
-
-
-
-
+    print('wradlib: Calling module <ipol> as main...')
